import pytest

import autofit.mapper.prior_model.deferred
from autofit import exc
from autofit.mapper import model_mapper as mm
from autofit.mapper import prior as p
from autofit.mapper.prior_model import prior_model as pm
from test import mock
from test.mock import DeferredClass


@pytest.fixture(name="deferred_instance")
def make_deferred_instance():
    prior_model = pm.PriorModel(mock.SimpleClass)
    prior_model.two = autofit.mapper.prior_model.deferred.DeferredArgument()

    return prior_model.instance_for_arguments(
        {
            prior_model.one: 1.0
        }
    )


class TestCase:
    def test_is_deferred(self, deferred_instance):
        assert isinstance(deferred_instance,
                          autofit.mapper.prior_model.deferred.DeferredInstance)

    def test_instantiate(self, deferred_instance):
        instance = deferred_instance(two=2.0)
        assert isinstance(instance, mock.SimpleClass)
        assert instance.one == 1.0
        assert instance.two == 2.0

    def test_deferred_exception(self, deferred_instance):
        with pytest.raises(exc.DeferredInstanceException):
            print(deferred_instance.one)

    def test_deferred_config(self):
        mapper = mm.ModelMapper()
        mapper.DeferredClass = DeferredClass

        assert mapper.prior_count == 1

        deferred_instance = mapper.instance_from_unit_vector([1.0]).DeferredClass

        assert isinstance(deferred_instance,
                          autofit.mapper.prior_model.deferred.DeferredInstance)

        instance = deferred_instance(two=2.0)

        assert isinstance(instance, mock.DeferredClass)
        assert instance.one == 1.0
        assert instance.two == 2.0



from copy import deepcopy

import pytest

import autofit.mapper.model
import autofit.mapper.prior_model.collection
import autofit.optimize.non_linear.multi_nest
import autofit.optimize.non_linear.non_linear
import test.mock
from autofit.mapper import model_mapper as mm
from autofit.mapper import prior as p
from autofit.mapper.prior_model import prior_model as pm


@pytest.fixture(name="prior_model")
def make_prior_model():
    return pm.PriorModel(test.mock.GeometryProfile)


class TestCase(object):
    def test_prior_model(self, prior_model):
        prior_model_copy = deepcopy(prior_model)
        assert prior_model == prior_model_copy

        prior_model_copy.centre_0 = p.UniformPrior()

        assert prior_model != prior_model_copy

    def test_list_prior_model(self, prior_model):
        list_prior_model = autofit.mapper.prior_model.collection.CollectionPriorModel([prior_model])
        list_prior_model_copy = deepcopy(list_prior_model)
        assert list_prior_model == list_prior_model_copy

        list_prior_model[0].centre_0 = p.UniformPrior()

        assert list_prior_model != list_prior_model_copy

    def test_model_mapper(self, prior_model):
        model_mapper = mm.ModelMapper()
        model_mapper.prior_model = prior_model
        model_mapper_copy = deepcopy(model_mapper)

        assert model_mapper == model_mapper_copy

        model_mapper.prior_model.centre_0 = p.UniformPrior()

        assert model_mapper != model_mapper_copy

    def test_non_trivial_equality(self):
        model_mapper = mm.ModelMapper()
        model_mapper.galaxy = test.mock.GalaxyModel(light_profile=test.mock.GeometryProfile, mass_profile=test.mock.GeometryProfile)
        model_mapper_copy = deepcopy(model_mapper)

        assert model_mapper == model_mapper_copy

        model_mapper.galaxy.light_profile.centre_0 = p.UniformPrior()

        assert model_mapper != model_mapper_copy

    def test_model_instance_equality(self):
        model_instance = autofit.mapper.model.ModelInstance()
        model_instance.profile = test.mock.GeometryProfile()
        model_instance_copy = deepcopy(model_instance)

        assert model_instance == model_instance_copy

        model_instance.profile.centre = (1., 2.)

        assert model_instance != model_instance_copy

    def test_non_linear_equality(self):
        nlo = autofit.optimize.non_linear.non_linear.NonLinearOptimizer("phase name")
        nlo.variable.profile = test.mock.GeometryProfile
        nlo_copy = deepcopy(nlo)

        assert nlo_copy == nlo

        nlo.variable.profile.centre_0 = p.UniformPrior()

        assert nlo_copy != nlo

    def test_multinest_equality(self):
        nlo = autofit.optimize.non_linear.multi_nest.MultiNest("phase name")
        nlo_copy = deepcopy(nlo)

        assert nlo == nlo_copy

        nlo.n_live_points += 1

        assert nlo != nlo_copy



import configparser

from autofit import conf


def format_string_for_label(label: str) -> str:
    """
    Get the format for the label. Attempts to extract the key string associated with
    the dimension. Seems dodgy.

    Parameters
    ----------
    label
        A string label

    Returns
    -------
    format
        The format string (e.g {:.2f})
    """
    label = label.split("_within_")[0]
    label = label.split("_at_")[0]
    try:
        return conf.instance.label_format.get("format", label)
    except configparser.NoOptionError:
        label = "_".join(label.split("_")[1:])
        if len(label) == 0:
            raise
        return format_string_for_label(label)


def label_and_label_string(label0, label1, whitespace):
    return label0 + label1.rjust(whitespace - len(label0) + len(label1))


def label_and_value_string(
        label,
        value,
        whitespace,
        format_string=None
):
    format_str = format_string or format_string_for_label(label)
    value = format_str.format(value)
    return label + value.rjust(whitespace - len(label) + len(value))


def label_value_and_limits_string(
        label,
        value,
        lower_limit,
        upper_limit,
        whitespace,
        format_string=None
):
    format_str = format_string or format_string_for_label(label)
    value = format_str.format(value)
    upper_limit = format_str.format(upper_limit)
    lower_limit = format_str.format(lower_limit)
    value = value + ' (' + lower_limit + ', ' + upper_limit + ')'
    return label + value.rjust(whitespace - len(label) + len(value))


def label_value_and_unit_string(
        label,
        value,
        unit,
        whitespace,
        format_string=None
):
    format_str = format_string or format_string_for_label(label)
    value = (format_str + ' {}').format(value, unit)
    return label + value.rjust(whitespace - len(label) + len(value))


def output_list_of_strings_to_file(file, list_of_strings):
    file = open(file, 'w')
    file.write(''.join(list_of_strings))
    file.close()
